from importlib import import_module
from pkgutil import iter_modules
from typing import Callable
from typing import List
from typing import Type

from cleo.commands.command import Command
from cleo.loaders.factory_command_loader import FactoryCommandLoader


class LazyCommandLoader(FactoryCommandLoader):
    def __init__(self, package: str) -> None:
        module = import_module(".console.commands", package)

        super(LazyCommandLoader, self).__init__(
            {
                name: self._load_command("{}.{}".format(module.__name__, name))
                for finder, name, ispkg in iter_modules(module.__path__)
            }
        )

    def _load_command(self, name: str) -> Callable:
        def _load() -> Type[Command]:
            command_class = getattr(import_module(name), "Command")
            return command_class()

        return _load



from cleo.application import Application as BaseApplication
from j2tpl import __name__, __version__
from j2tpl.loaders.lazy_command_loader import LazyCommandLoader


class Application(BaseApplication):
    def __init__(self) -> None:
        super(Application, self).__init__(__name__, __version__)

        self.set_display_name(__name__)
        self.set_command_loader(LazyCommandLoader(__name__))


def main() -> int:
    return Application().run()


if __name__ == "__main__":
    main()



import json
from pathlib import Path
from shutil import copy
from typing import Dict
from typing import List

import cleo
from cleo.commands.command import Command as BaseCommand
from jinja2 import Environment
from jinja2 import FileSystemLoader


class Command(BaseCommand):
    """
    Initialises target using a template

    init
        {source : Source template file or folder}
        {target : Target file or folder}
        {--f|force : Overwrite existing file(s)}
        {--x|exclude=* : Excluded path(s)}
        {--var=* : Variable in <comment>key=value</comment> format}
        {--var-file= : Variable file}
    """

    def handle(self) -> None:
        source = Path(self.argument("source"))
        target = Path(self.argument("target"))
        self._force = self.option("force")
        self._excludes = [x for x in self.option("exclude")]
        self._args = self._load_vars_from_file(self.option("var-file"))

        for o in self.option("var"):
            k, v = tuple(o.split("="))
            self._args[k] = v

        self._process_path(source, target)

    def _load_vars_from_file(self, var: Path) -> Dict[str, str]:
        data = {}
        if var:
            with open(var, "r") as f:
                data = json.loads(f.read())
        return data

    def _process_path(self, source: Path, target: Path) -> None:
        for p in source.iterdir():
            if p.name in self._excludes:
                self.line(f"'{p}' excluded source path")
                continue

            t = target / p.relative_to(source)
            if p.is_file():
                if t.exists() and not self._force:
                    self.line(f"'{t}' already exists")
                    # , verbosity=cleo.VERBOSITY_NORMAL)
                    continue

                if p.suffix == ".j2":
                    env = Environment(
                        loader=FileSystemLoader(p.parent),
                        # extensions=?
                        keep_trailing_newline=True,
                    )
                    with open(t.parent / t.stem, "w") as out:
                        tmpl = env.get_template(p.name)
                        out.write(tmpl.render(self._args))
                else:
                    copy(p, t)
            elif p.is_dir():
                if not t.exists():
                    t.mkdir()
                self._process_path(p, t)



import json

from pathlib import Path
from typing import Any
from typing import Dict


class JsonParser:
    def __init__(self) -> None:
        pass

    def parse_file(self, path: Path) -> Dict[str, Any]:
        d = {}
        with open(path, "r") as f:
            d = json.loads(f.read())
        return d



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['j2tpl',
 'j2tpl.console',
 'j2tpl.console.commands',
 'j2tpl.loaders',
 'j2tpl.parsers']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=2.11.3,<3.0.0', 'cleo==1.0.0a2']

entry_points = \
{'console_scripts': ['j2tpl = j2tpl.console.application:main']}

setup_kwargs = {
    'name': 'j2tpl',
    'version': '2021.3.1',
    'description': '',
    'long_description': None,
    'author': 'Janne K',
    'author_email': '0x022b@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
